import tkinter as tk
from tkinter import ttk
from tkinter import filedialog
import tkinter.messagebox as msg
import threading
import time
import sys

# To avoid recursion overflow
# Maximum time for listening: 10 min
sys.setrecursionlimit(6000)


class ExtractThread(threading.Thread):
    def __init__(self, input_dir, output_dir, start, end, step):
        super().__init__()
        self._input_dir = input_dir
        self._output_dir = output_dir
        self._model = None
        self._error = None
        self._start = start
        self._end = end
        self._step = step

    def run(self):
        self.check_sign()
        if 'Amid1Extract' not in dir():
            from amid1_extract import Amid1Extract
        try:
            self._model = Amid1Extract(input_dir=self._input_dir, output_dir=self._output_dir)
            self._model(start=self._start, end=self._end, step=self._step)
        except Exception as e:
            self._error = e

    def check_sign(self):
        # Check the sign of step
        if self._start > self._end and self._step > 0:
            self._step = -self._step
        elif self._start < self._end and self._step < 0:
            self._step = -self._step


# Predict by AI
class PredThread(threading.Thread):
    def __init__(self, input_dir, output_dir, model='Standard', exclude_trash=True):
        # File loading thread
        super().__init__()
        self._input_dir = input_dir
        self._output_dir = output_dir
        self._exclude_trash = exclude_trash
        self._model = model
        self._error = None

    def run(self):
        if 'Amid1Pred' not in dir():
            from amid1_pred_g import Amid1Pred
            global Amid1Pred

        try:
            self._model = Amid1Pred(input_dir=self._input_dir, output_dir=self._output_dir, model=self._model)
            self._model(exclude_trash=self._exclude_trash)  # Model function will be added later
        except Exception as e:
            self._error = e


# Gaussian Fit
class FitThread(threading.Thread):
    def __init__(self, input_dir, output_dir, exclude_area=True, exclude_trash=False, parallel=False, maxerror=0.07):
        super().__init__()
        self._input_dir = input_dir
        self._output_dir = output_dir
        self._exclude_area = exclude_area
        self._model = None
        self._error = None
        self._parallel = parallel
        self._exclude_trash = exclude_trash
        self._maxerror = maxerror

    def run(self):
        if 'Amid1Fit' not in dir():
            from amid1_peakfit_g import Amid1Fit
            global Amid1Fit
        try:
            self._model = Amid1Fit(input_dir=self._input_dir, output_dir=self._output_dir,
                                   exclude_area=self._exclude_area,
                                   exclude_trash=self._exclude_trash,
                                   maxerror=self._maxerror)(self._parallel)
        except Exception as e:
            self._error = e


class ClusterThread(threading.Thread):
    # Thread for clustering
    def __init__(self, input_dir, output_dir, cats, newdir, outputopt):
        # outputopt <- tuple (bool(clusters?), bool(visualization?), bool(eigenvectors?))
        super().__init__()

        self._input_dir = input_dir
        self._output_dir = output_dir
        self._model = None
        self._error = None
        self._cats = cats
        self._newdir = newdir
        self._outputopt = outputopt

    def run(self):
        if 'Clustering' not in dir():
            from cluster import Clustering
            global Clustering
        try:
            self._model = Clustering(input_dir=self._input_dir,
                                     output_dir=self._output_dir,
                                     cats=self._cats)
            self._model.getdata()
            if self._newdir:
                self._model.creat_dir()
            if self._outputopt[0] or self._outputopt[1]:
                self._model.clustering()
            if self._outputopt[1] or self._outputopt[2]:
                self._model.pca()
            if self._outputopt[0]:
                self._model.output_clusters()
            if self._outputopt[1]:
                self._model.plot_clusters()
            if self._outputopt[2]:
                self._model.plot_eigen()


        except Exception as e:
            self._error = e


# 2D Reconstruct
class ReconstructThread(threading.Thread):
    # Thread for making 2D distribution plots
    def __init__(self, input_dir, output_dir, order, shape, plot, mat):
        super().__init__()

        self._input_dir = input_dir
        self._output_dir = output_dir
        self._order = order
        self._shape = shape
        self._plot = plot
        self._mat = mat
        self._model = None
        self._error = None

    def run(self):
        if 'Phase2D' not in dir():
            from phase2d import Phase2D
            global Phase2D
        try:
            self._model = Phase2D(input_dir=self._input_dir,
                                  output_dir=self._output_dir,
                                  order=self._order,
                                  shape=self._shape,
                                  plot=self._plot,
                                  mat=self._mat)
            self._model()
        except Exception as e:
            self._error = e


# Public listener to threads
class Listener(threading.Thread):
    def __init__(self):
        super().__init__()
        self._thread = None
        # Set commands to the listener so that it can execute automatically
        self._errcommand = None
        self._fincommand = None
        self._start_time = None
        self._end_time = None

    def set_stats(self, thread, errcommand=None, fincommand=None):
        self._thread = thread
        self._errcommand = errcommand
        self._fincommand = fincommand

    def run(self):
        def listen():
            if self._thread._error:
                self._errcommand()
                print('[Line {}]'.format(self._thread._error.__traceback__.tb_lineno), self._thread._error)
                self.__init__()
            elif self._thread.is_alive():
                self.after(0.1, listen)
            else:
                self._end_time = time.time()
                self._fincommand()
                self.__init__()

        self._start_time = time.time()
        self._thread.start()
        listen()

    def after(self, waittime, command):
        # Write an "after" function similar to the one from tk
        # Note that the time unit here is second rather than millisecond
        time.sleep(waittime)
        command()

    def calculate_time(self):
        return round(self._end_time - self._start_time, 1)

    @property
    def thread(self):
        return self._thread


listener = Listener()


class MainApp(tk.Tk):
    def __init__(self):
        super().__init__()

        self.geometry('600x400')
        self.title('Amid1Pred')
        self.f1 = HeadFrame(self)
        self.f1.pack()

        # Notebook layout
        self.notebook = ttk.Notebook(self)

        self.nf4 = ExtractFrame(self.notebook)
        self.nf4.pack()
        self.nf1 = PredFrame(self.notebook)
        self.nf1.pack()
        self.nf2 = FitFrame(self.notebook)
        self.nf2.pack()
        self.nf5 = ClusterFrame(self.notebook)
        self.nf5.pack()
        self.nf3 = DistributionFrame(self.notebook)
        self.nf3.pack()

        self.notebook.add(self.nf4, text='Extract Local')
        self.notebook.add(self.nf1, text='Use AI')
        self.notebook.add(self.nf2, text='Gaussian Fit')
        self.notebook.add(self.nf5, text='Clustering')
        self.notebook.add(self.nf3, text='2D Distribution')

        self.notebook.pack()

        self.f2 = self.set_status_frame()
        self.f3 = AuthorBar(self)
        self.f3.pack(side=tk.BOTTOM)
        self.f2.pack(side=tk.BOTTOM)

    def set_status_frame(self):
        mainf = tk.Frame(self, width=580, height=50, pady=10)
        mainf.pack_propagate(0)
        f1 = tk.Frame(mainf, width=120, height=30)
        f1.pack_propagate(0)
        l1 = tk.Label(f1, text='Status:')
        l1.pack(side=tk.LEFT)
        f1.pack(side=tk.LEFT)

        f2 = tk.Frame(mainf, width=460, height=30, bg='white')
        f2.pack_propagate(0)
        self.statusbar = tk.Label(f2, text='Standby.', bg='white', fg='#00BB00')
        self.statusbar.pack(side=tk.LEFT)
        f2.pack(side=tk.RIGHT)

        return mainf

    def change_status(self, msg, bg, fg):
        self.statusbar['text'] = msg
        self.statusbar['bg'] = bg
        self.statusbar['fg'] = fg


class HeadFrame(tk.Frame):
    def __init__(self, master):
        super().__init__()
        self['width'] = 580
        self['height'] = 60
        self['bg'] = '#DDFFDD'
        self['pady'] = 15
        self.pack_propagate(0)

        self.l1 = tk.Label(self, text='Solution to Phase Contents', fg='black', bg='#DDFFDD', font=(None, 15))
        self.l1.pack()


class StatusFrame(tk.Frame):
    def __init__(self, master):
        super().__init__()
        self['width'] = 580
        self['height'] = 50
        self['pady'] = 10
        self.pack_propagate(0)

        self.f1 = tk.Frame(self, width=120, height=30)
        self.f1.pack_propagate(0)
        self.l1 = tk.Label(self.f1, text='Status:')
        self.l1.pack(side=tk.LEFT)
        self.f1.pack(side=tk.LEFT)

        self.f2 = tk.Frame(self, width=460, height=30, bg='white')
        self.f2.pack_propagate(0)
        self.l2 = tk.Label(self.f2, text='Standby.', bg='white', fg='#00BB00')
        self.l2.pack(side=tk.LEFT)
        self.f2.pack(side=tk.RIGHT)

    def change_status(self, msg, bg, fg):
        self.l2['text'] = msg
        self.l2['bg'] = bg
        self.l2['fg'] = fg


class AuthorBar(tk.Frame):
    # Author information
    def __init__(self, master):
        super().__init__()
        self['width'] = 380
        self['height'] = 20
        self['pady'] = 5

        self.l1 = tk.Label(self, text='Mingrui Jiang, Biomateriomics Group, ShanghaiTech, 2020. Version 0.3.2',
                           fg='black', font=(None, 9))
        self.l1.pack()


class NoteFrame(tk.Frame):
    # Base framework of notebook
    # Note that reference error may occur if subordinate frames are defined as "class" belonging to tk.Frame
    # Therefore, functions returning frames are defined

    def __init__(self):
        super().__init__()

        self['width'] = 580
        self['height'] = 240
        self.pack_propagate(0)

        # Property constant which is helpful for some operations
        self._default_output_head = 'Result_of_'

        # Because multiple widgets may target textboxes, they are set as interface
        self.t_input = None
        self.t_output = None

    def set_input_frame(self):

        def fill_input(event=None):
            # Select and open file
            # Only callable when
            selected = filedialog.askopenfilename()
            if selected:
                self.t_input.delete(1.0, 1023.0)
                self.t_input.insert(1.0, selected)

        mainf = tk.Frame(self, width=580, height=45, pady=10)
        mainf.pack_propagate(0)
        f1 = tk.Frame(mainf, width=80, height=25)
        f1.pack_propagate(0)
        l = tk.Label(f1, text='Input File', fg='black')
        l.pack()
        f1.pack(side=tk.LEFT)

        f2 = tk.Frame(mainf, width=420, height=25)
        f2.pack_propagate(0)
        self.t_input = tk.Text(f2, fg='black', font=(None, 12))
        self.t_input.insert(1.0, '*.csv')
        self.t_input.pack()
        f2.pack(side=tk.LEFT)

        f3 = tk.Frame(mainf, width=80, height=25)
        f3.pack_propagate(0)
        b = tk.Button(f3, text='Browse', bg='#FFFFDD', fg='black', width=60)
        b.pack()
        b.bind('<Button-1>', fill_input)
        f3.pack(side=tk.RIGHT)

        return mainf

    def set_output_frame(self):
        # Must call self.gene_input_frame before

        mainf = tk.Frame(self, width=580, height=45, pady=10)
        mainf.pack_propagate(0)

        f1 = tk.Frame(mainf, width=80, height=25)
        f1.pack_propagate(0)
        l = tk.Label(f1, text='Output File', fg='black')
        l.pack()
        f1.pack(side=tk.LEFT)

        f2 = tk.Frame(mainf, width=420, height=25)
        f2.pack_propagate(0)
        self.t_output = tk.Text(f2, fg='black', font=(None, 12))
        self.t_output.insert(1.0, '*.csv')
        self.t_output.pack()
        f2.pack(side=tk.LEFT)

        f3 = tk.Frame(mainf, width=80, height=25)
        f3.pack_propagate(0)
        b = tk.Button(f3, text='Default', bg='#FFFFDD', fg='black', width=60)
        b.pack()
        b.bind('<Button-1>', self.fill_output)
        f3.pack(side=tk.RIGHT)

        return mainf

    def fill_output(self, event=None):

        tx = self.t_input.get(1.0, 1023.0).strip('\n')
        # IMPORTANT NOTICE: get() returns a string ending with '\n'
        if len(tx) < 4 or tx.strip('\n')[-4:] != '.csv':
            msg.showerror('Invalid Input', 'Please select an input csv file.')
        else:
            self.t_output.delete(1.0, 1023.0)
            self.t_output.insert(1.0, tx[:tx.rfind('/') + 1] + self._default_output_head + tx.split('/')[-1])

    def set_outdir_frame(self):
        # Set output directory
        # ONLY for those frames do not need set_output_frame
        mainf = tk.Frame(self, width=580, height=45, pady=10)
        mainf.pack_propagate(0)

        f1 = tk.Frame(mainf, width=80, height=25)
        f1.pack_propagate(0)
        l = tk.Label(f1, text='Output Dir', fg='black')
        l.pack()
        f1.pack(side=tk.LEFT)

        f2 = tk.Frame(mainf, width=420, height=25)
        f2.pack_propagate(0)
        self.t_output = tk.Text(f2, fg='black', font=(None, 12))
        self.t_output.insert(1.0, '*')
        self.t_output.pack()
        f2.pack(side=tk.LEFT)

        f3 = tk.Frame(mainf, width=80, height=25)
        f3.pack_propagate(0)
        b = tk.Button(f3, text='Default', bg='#FFFFDD', fg='black', width=60)
        b.pack()
        b.bind('<Button-1>', self.fill_outdir)
        f3.pack(side=tk.RIGHT)

        return mainf

    def fill_outdir(self, event=None):
        # Autofill output directory option, similar to fill_output
        tx = self.t_input.get(1.0, 1023.0).strip('\n')
        if len(tx) < 4 or tx.strip('\n')[-4:] != '.csv':
            msg.showerror('Invalid Input', 'Please select an input csv file.')
        else:
            self.t_output.delete(1.0, 1023.0)
            self.t_output.insert(1.0, tx[:tx.rfind('/') + 1] + self._default_output_head + tx.split('/')[-1][:-4])

    def set_doh(self, newstr):
        # Changing self._default_output_head
        self._default_output_head = newstr


class ExtractFrame(NoteFrame):
    # Extract Amid1 region from the whole spectrum
    def __init__(self, master):
        super().__init__()
        self.set_doh('Local_')

        self.input_frame = self.set_input_frame()
        self.input_frame.pack()
        self.output_frame = self.set_output_frame()
        self.output_frame.pack()
        self.config_frame = self.set_config_frame()
        self.config_frame.pack()

        # The button frame
        self.f1 = tk.Frame(self, width=580, height=150, pady=40)
        self.f1.pack_propagate(0)
        self.b = tk.Button(self.f1, text='Extract Data', height=30, bg='#FFFFDD', fg='black')
        self.b.bind('<Button-1>', self.extract)
        self.b.pack(fill=tk.X)
        self.f1.pack()

    def extract(self, event=None):
        if not listener.thread:
            if self.t_output.get(1.0, 1023.0).strip('\n') == '*.csv':
                self.fill_output()
            app.change_status('Running Extraction...', 'white', '#BBBB00')
            listener.set_stats(thread=ExtractThread(input_dir=self.t_input.get(1.0, 1023.0).strip('\n'),
                                                    output_dir=self.t_output.get(1.0, 1023.0).strip('\n'),
                                                    start=float(self.t1.get(1.0, 1023.0)),
                                                    end=float(self.t2.get(1.0, 1023.0)),
                                                    step=float(self.t3.get(1.0, 1023.0))),
                               errcommand=lambda: app.change_status(str(listener.thread._error), 'white', '#BB0000'),
                               fincommand=lambda: app.change_status(
                                   'Finished. Time Used: ' + str(listener.calculate_time()) + ' s',
                                   'white', '#00BB00'))
            listener.start()

    def set_config_frame(self):
        mainf = tk.Frame(self, width=580, height=40, pady=10)
        mainf.pack_propagate(0)

        f1 = tk.Frame(mainf, width=60, height=30)
        f1.pack_propagate(0)
        l1 = tk.Label(f1, text='Start:')
        l1.pack()
        f1.pack(side=tk.LEFT)

        f2 = tk.Frame(mainf, width=80, height=30)
        f2.pack_propagate(0)
        self.t1 = tk.Text(f2, fg='black', font=(None, 12))
        self.t1.pack(fill=tk.X)
        f2.pack(side=tk.LEFT)

        f3 = tk.Frame(mainf, width=60, height=30)
        f3.pack_propagate(0)
        l2 = tk.Label(f3, text='End:')
        l2.pack()
        f3.pack(side=tk.LEFT)

        f4 = tk.Frame(mainf, width=80, height=30)
        f4.pack_propagate(0)
        self.t2 = tk.Text(f4, fg='black', font=(None, 12))
        self.t2.pack(fill=tk.X)
        f4.pack(side=tk.LEFT)

        f5 = tk.Frame(mainf, width=60, height=30)
        f5.pack_propagate(0)
        l3 = tk.Label(f5, text='Step:')
        l3.pack()
        f5.pack(side=tk.LEFT)

        f6 = tk.Frame(mainf, width=80, height=30)
        f6.pack_propagate(0)
        self.t3 = tk.Text(f6, fg='black', font=(None, 12))
        self.t3.pack(fill=tk.X)
        f6.pack(side=tk.LEFT)

        return mainf


class PredFrame(NoteFrame):
    def __init__(self, master):
        super().__init__()

        # Initialize public widgets
        self.cb = None
        self.v1 = tk.IntVar()
        self.v1.set(1)
        self.set_doh('AI_Prediction_of_')

        self.input_frame = self.set_input_frame()
        self.input_frame.pack()
        self.output_frame = self.set_output_frame()
        self.output_frame.pack()
        self.cfg_frame = self.set_cfg_frame()
        self.cfg_frame.pack()
        self.pred_frame = self.set_pred_frame()
        self.pred_frame.pack()

    def set_cfg_frame(self):
        # Frame for configuration to prediction
        mainf = tk.Frame(self, width=580, height=45, pady=10)
        mainf.pack_propagate(0)

        f1 = tk.Frame(mainf, width=80, height=25)
        f1.pack_propagate(0)
        l1 = tk.Label(f1, text='Model', fg='black')
        l1.pack()
        f1.pack(side=tk.LEFT)

        f2 = tk.Frame(mainf, width=120, height=25, padx=20)
        f2.pack_propagate(0)
        self.cb = ttk.Combobox(f2)
        self.cb['values'] = ['Standard', 'Light']
        self.cb.current(0)
        self.cb.pack(fill=tk.X)
        f2.pack(side=tk.LEFT)

        f3 = tk.Frame(mainf, width=160, height=25, padx=20)
        f3.pack_propagate(0)
        ch = tk.Checkbutton(f3, text='Autodetect Trash', variable=self.v1)
        ch.pack()
        f3.pack(side=tk.LEFT)

        return mainf

    def set_pred_frame(self):
        # Prediction button frame
        mainf = tk.Frame(self, width=580, height=60, pady=15)
        mainf.pack_propagate(0)
        b = tk.Button(mainf, width=580, text='Predict by AI', bg='#FFFFDD', fg='black')
        b.bind('<Button-1>', self.aipred)
        b.pack(fill=tk.X)

        return mainf

    def aipred(self, event=None):
        # Predict by AI app

        # If thread exists, do not run
        if not listener.thread:
            # Inform the problem of the thread
            if self.t_output.get(1.0, 1023.0).strip('\n') == '*.csv':
                self.fill_output()
            app.change_status('Running Prediction...', 'white', '#BBBB00')
            # Listen to execution status
            listener.set_stats(thread=PredThread(input_dir=self.t_input.get(1.0, 1023.0).strip('\n'),
                                                 output_dir=self.t_output.get(1.0, 1023.0).strip('\n'),
                                                 exclude_trash=self.v1.get(), model=self.cb.get()),
                               errcommand=lambda: app.change_status(listener.thread._error, 'white', '#BB0000'),
                               fincommand=lambda: app.change_status(
                                   'Finished. Time Used: ' + str(listener.calculate_time()) + ' s',
                                   'white', '#00BB00'))
            listener.start()


class FitFrame(NoteFrame):
    def __init__(self, master):
        super().__init__()

        self.set_doh('Gaussian_Fit_Result_of_')
        self.v = tk.BooleanVar()
        self.v.set(False)
        self.v2 = tk.BooleanVar()
        self.v2.set(False)

        self.input_frame = self.set_input_frame()
        self.input_frame.pack()
        self.output_frame = self.set_output_frame()
        self.output_frame.pack()
        self.cfg_frame = self.set_cfg_frame()
        self.cfg_frame.pack()
        self.cfg2_frame = self.set_cfg2_frame()
        self.cfg2_frame.pack()
        self.fit_frame = self.set_fit_frame()
        self.fit_frame.pack()

    def set_cfg_frame(self):
        mainf = tk.Frame(self, width=580, height=45, pady=10)
        mainf.pack_propagate(0)
        f1 = tk.Frame(mainf, width=140, height=25)
        f1.pack_propagate(0)
        ch = tk.Checkbutton(f1, text='Area Values', fg='black', variable=self.v)
        ch.pack(side=tk.TOP)
        f1.pack(side=tk.LEFT)

        f2 = tk.Frame(mainf, width=160, height=25, padx=20)
        f2.pack_propagate(0)
        ch2 = tk.Checkbutton(f2, text='Autodetect trash', fg='black', variable=self.v2)
        ch2.pack()
        f2.pack(side=tk.LEFT)

        f3 = tk.Frame(mainf, width=100, height=25, padx=10)
        f3.pack_propagate(0)
        l1 = tk.Label(f3, text='Parallelism:', fg='black')
        l1.pack(side=tk.TOP)
        f3.pack(side=tk.LEFT)

        f4 = tk.Frame(mainf, width=90, height=25)
        f4.pack_propagate(0)
        self.cb = ttk.Combobox(f4)
        self.cb['values'] = ['Disabled', '4']
        self.cb.current(0)
        self.cb.pack(side=tk.TOP, fill=tk.X)
        f4.pack(side=tk.LEFT)

        return mainf

    def set_cfg2_frame(self):
        mainf = tk.Frame(self, width=580, height=45, pady=10)
        mainf.pack_propagate(0)
        f1 = tk.Frame(mainf, width=180, height=25)
        f1.pack_propagate(0)
        l1 = tk.Label(f1, text='Maximum RMSE Allowed:', fg='black')
        l1.pack(side=tk.TOP)
        f1.pack(side=tk.LEFT)

        f2 = tk.Frame(mainf, width=90, height=25)
        f2.pack_propagate(0)
        self.cb2 = ttk.Combobox(f2)
        self.cb2['values'] = ['1.00', '0.10', '0.07', '0.05', '0.03', '0.02']
        self.cb2.current(0)
        self.cb2.pack(side=tk.TOP, fill=tk.X)
        f2.pack(side=tk.LEFT)

        return mainf

    def set_fit_frame(self):
        mainf = tk.Frame(self, width=580, height=60, pady=15)
        mainf.pack_propagate(0)
        b = tk.Button(mainf, text='Gaussian Fit', bg='#FFFFDD', fg='black')
        b.bind('<Button-1>', self.peakfit)
        b.pack(side=tk.TOP, fill=tk.X)

        return mainf

    def peakfit(self, event=None):
        if not listener._thread:
            if self.t_output.get(1.0, 1023.0).strip('\n') == '*.csv':
                self.fill_output()
            try:
                parallel = int(self.cb.get())
            except ValueError:
                parallel = False
            app.change_status('Running Gaussian Fit...', 'white', '#BBBB00')
            listener.set_stats(thread=FitThread(input_dir=self.t_input.get(1.0, 1023.0).strip('\n'),
                                                output_dir=self.t_output.get(1.0, 1023.0).strip('\n'),
                                                exclude_area=1 - self.v.get(), # Negation is needed
                                                exclude_trash=self.v2.get(),
                                                maxerror = float(self.cb2.get()),
                                                parallel=parallel),
                               errcommand=lambda: app.change_status(listener.thread._error, 'white',
                                                                    '#BB0000'),
                               fincommand=lambda: app.change_status(
                                   'Finished. Time Used: ' + str(listener.calculate_time()) + ' s',
                                   'white', '#00BB00'))
            listener.start()


class ClusterFrame(NoteFrame):
    # Frame for clustering and PCA analysis
    def __init__(self, master):
        super().__init__()
        self.cb = None  # Number of clusters
        self.v1 = tk.BooleanVar()  # Creat directory
        self.v1.set(True)
        self.v2 = tk.BooleanVar()  # Output cluster
        self.v2.set(True)
        self.v3 = tk.BooleanVar()  # Output cluster distribution
        self.v3.set(True)
        self.v4 = tk.BooleanVar()  # Output PCA eigenvectors
        self.v4.set(True)

        self.set_doh('Clustering_')
        self.input_frame = self.set_input_frame()
        self.input_frame.pack()
        self.outdir_frame = self.set_outdir_frame()
        self.outdir_frame.pack()
        self.cfg_frame = self.set_cfg_frame()
        self.cfg_frame.pack()
        self.exec_frame = self.set_exec_frame()
        self.exec_frame.pack()

    def set_cfg_frame(self):
        mainf = tk.Frame(self, width=580, height=90, pady=10)
        mainf.pack_propagate(0)
        f1 = tk.Frame(mainf, width=580, height=25)
        f1.pack_propagate(0)
        f11 = tk.Frame(f1, width=220, height=25)
        f11.pack_propagate(0)
        l1 = tk.Label(f11, text='Number of Clusters: ', fg='black', padx=10)
        l1.pack(side=tk.LEFT)
        f11.pack(side=tk.LEFT)

        f12 = tk.Frame(f1, width=360, height=25)
        f12.pack_propagate(0)
        self.cb = ttk.Combobox(f12)
        self.cb['values'] = ['2', '3', '4', '5', '6']
        self.cb.current(0)
        self.cb.pack(fill=tk.X)
        f12.pack(side=tk.RIGHT)

        f1.pack()

        f2 = tk.Frame(mainf, width=580, height=45, pady=10)
        f2.pack_propagate(0)
        f21 = tk.Frame(f2, width=120, height=25)
        f21.pack_propagate(0)
        l2 = tk.Label(f21, text='Output Options: ', fg='black')
        l2.pack()
        f21.pack(side=tk.LEFT)

        f22 = tk.Frame(f2, width=115, height=25)
        f22.pack_propagate(0)
        ch1 = tk.Checkbutton(f22, text='New Dir', fg='black', variable=self.v1)
        ch1.pack()
        f22.pack(side=tk.LEFT)

        f23 = tk.Frame(f2, width=115, height=25)
        f23.pack_propagate(0)
        ch2 = tk.Checkbutton(f23, text='Clusters (.csv)', fg='black', variable=self.v2)
        ch2.pack()
        f23.pack(side=tk.LEFT)

        f24 = tk.Frame(f2, width=115, height=25)
        f24.pack_propagate(0)
        ch3 = tk.Checkbutton(f24, text='Visual (.png)', fg='black', variable=self.v3)
        ch3.pack()
        f24.pack(side=tk.LEFT)

        f25 = tk.Frame(f2, width=115, height=25)
        f25.pack_propagate(0)
        ch4 = tk.Checkbutton(f25, text='Eigen (.png)', fg='black', variable=self.v4)
        ch4.pack()
        f25.pack(side=tk.LEFT)

        f2.pack()

        return mainf

    def set_exec_frame(self):
        mainf = tk.Frame(self, width=580, height=30)
        mainf.pack_propagate(0)
        b = tk.Button(mainf, width=580, text='Do Clustering/PCA Analysis', bg='#FFFFDD', fg='black')
        b.bind('<Button-1>', self.cluster)
        b.pack(fill=tk.X)

        return mainf

    def cluster(self, event=None):
        if not listener.thread:
            if self.t_output.get(1.0, 1023.0).strip('\n') == '*':
                self.fill_outdir()
            app.change_status('Running Clustering...', 'white', '#BBBB00')
            listener.set_stats(thread=ClusterThread(input_dir=self.t_input.get(1.0, 1023.0).strip('\n'),
                                                    output_dir=self.t_output.get(1.0, 1023.0).strip('\n'),
                                                    cats=int(self.cb.get()),
                                                    newdir = self.v1.get(),
                                                    outputopt=(self.v2.get(), self.v3.get(), self.v4.get())),
                               errcommand=lambda: app.change_status(listener.thread._error, 'white', '#BB0000'),
                               fincommand=lambda: app.change_status(
                                   'Finished. Time Used: ' + str(listener.calculate_time()) + ' s',
                                   'white', '#00BB00'))
            listener.start()


class DistributionFrame(NoteFrame):
    # Frame for constructing and showing 2D distribution of phases
    def __init__(self, master):
        super().__init__()

        self.set_doh('2D_')
        self.t1 = None  # Size x
        self.t2 = None  # Size y
        self.cb2 = None  # Order
        self.v1 = tk.BooleanVar()  # Output Matrix
        self.v1.set(True)
        self.v2 = tk.BooleanVar()  # Output Plots
        self.v2.set(True)

        self.input_frame = self.set_input_frame()
        self.input_frame.pack()
        self.outdir_frame = self.set_outdir_frame()
        self.outdir_frame.pack()
        self.cfg_frame = self.set_cfg_frame()
        self.cfg_frame.pack()
        self.exec_frame = self.set_exec_frame()
        self.exec_frame.pack()

    def set_cfg_frame(self):
        # Config frame
        mainf = tk.Frame(self, width=580, height=90, pady=10)
        mainf.pack_propagate(0)
        f1 = tk.Frame(mainf, width=580, height=25)
        f1.pack_propagate(0)
        f11 = tk.Frame(f1, width=290, height=25)
        f11.pack_propagate(0)
        l1 = tk.Label(f11, text='Reconstruction Options:', fg='black', padx=10)
        l1.pack(side=tk.LEFT)
        f11.pack(side=tk.LEFT)

        f12 = tk.Frame(f1, width=290, height=25)
        f12.pack_propagate(0)
        l2 = tk.Label(f12, text='Output Options:', fg='black', padx=10)
        l2.pack(side=tk.LEFT)
        f12.pack(side=tk.LEFT)

        f1.pack()

        f2 = tk.Frame(mainf, width=580, height=45, pady=10)
        f2.pack_propagate(0)
        f21 = tk.Frame(f2, width=70, height=25)
        f21.pack_propagate(0)
        l3 = tk.Label(f21, text='Size: ', fg='black', padx=10)
        l3.pack(side=tk.LEFT)
        f21.pack(side=tk.LEFT)

        f22 = tk.Frame(f2, width=28, height=25)
        f22.pack_propagate(0)
        self.t1 = tk.Text(f22, fg='black', font=(None, 12))
        self.t1.insert(1.0, '64')
        self.t1.pack()
        f22.pack(side=tk.LEFT)

        f23 = tk.Frame(f2, width=10, height=25, padx=2)
        f23.pack_propagate(0)
        l4 = tk.Label(f23, text='×', fg='black')
        l4.pack()
        f23.pack(side=tk.LEFT)

        f24 = tk.Frame(f2, width=28, height=25)
        f24.pack_propagate(0)
        self.t2 = tk.Text(f24, fg='black', font=(None, 12))
        self.t2.pack()
        self.t2.insert(1.0, '64')
        f24.pack(side=tk.LEFT)

        f25 = tk.Frame(f2, width=70, height=25)
        f25.pack_propagate(0)
        l5 = tk.Label(f25, text='Order: ', fg='black', padx=10)
        l5.pack(side=tk.LEFT)
        f25.pack(side=tk.LEFT)

        f26 = tk.Frame(f2, width=80, height=25)
        f26.pack_propagate(0)
        self.cb2 = ttk.Combobox(f26)
        self.cb2['values'] = ['Regular', 'Alphabetical']
        self.cb2.current(0)
        self.cb2.pack(fill=tk.X)
        f26.pack(side=tk.LEFT)

        f27 = tk.Frame(f2, width=140, height=25)
        f27.pack_propagate(0)
        ch1 = tk.Checkbutton(f27, text='Matrix File (.mat)', variable=self.v1, padx=10)
        ch1.pack(side=tk.LEFT)
        f27.pack(side=tk.LEFT)

        f28 = tk.Frame(f2, width=150, height=25)
        f28.pack_propagate(0)
        ch2 = tk.Checkbutton(f28, text='Image Files (.png)', variable=self.v2, padx=10)
        ch2.pack(side=tk.LEFT)
        f28.pack(side=tk.LEFT)

        f2.pack()

        return mainf

    def set_exec_frame(self):
        mainf = tk.Frame(self, width=580, height=30)
        mainf.pack_propagate(0)
        b = tk.Button(mainf, width=580, text='Reconstruct 2D Distributions', bg='#FFFFDD', fg='black')
        b.bind('<Button-1>', self.reconstruct)
        b.pack(fill=tk.X)

        return mainf

    def reconstruct(self, event=None):
        if not listener.thread:
            # Warn the user if no output is selected
            if not (self.v1.get() | self.v2.get()):
                msg.showerror('Invalid Output types', 'Please select at least one type of outputs.')
                return

            if self.t_output.get(1.0, 1023.0).strip('\n') == '*':
                self.fill_outdir()

            # Parameter Determination
            if self.cb2.get() == 'Regular':
                order = 'r'
            else:
                order = 'a'

            try:
                shape = (int(self.t1.get(1.0, 15.0)), int(self.t2.get(1.0, 15.0)))
            except ValueError:
                msg.showerror('Invalid Input', 'Shape must be numbers.')
                return

            app.change_status('Running Reconstructions...', 'white', '#BBBB00')
            # Listen to execution status
            listener.set_stats(thread=ReconstructThread(input_dir=self.t_input.get(1.0, 1023.0).strip('\n'),
                                                        output_dir=self.t_output.get(1.0, 1023.0).strip('\n'),
                                                        order=order,
                                                        shape=shape,
                                                        plot=self.v2.get(),
                                                        mat=self.v1.get()),
                               errcommand=lambda: app.change_status(listener.thread._error, 'white', '#BB0000'),
                               fincommand=lambda: app.change_status(
                                   'Finished. Time Used: ' + str(listener.calculate_time()) + ' s',
                                   'white', '#00BB00'))
            listener.start()

if __name__ == '__main__':
    app = MainApp()
    app.mainloop()
