import numpy as np
from scipy import sparse
from scipy.sparse.linalg import spsolve
from scipy.optimize import curve_fit
from scipy.integrate import quad
import csv
import os
import sys
from stdspect import *


#def baseline_als(axis_y):
    #start_intensity = axis_y[0]
    #end_intensity = axis_y[-1]
    #cd = (end_intensity - start_intensity) / (len(axis_y) - 1)
    #return np.array([start_intensity+i*cd for i in range(len(axis_y))])

def baseline_als(y, lam=1e7, p=1e-3, niter=10):
    global z
    L = len(y)

    D = sparse.csc_matrix(np.diff(np.eye(L), 2))
    w = np.ones(L)
    for i in range(niter):
        W = sparse.spdiags(w, 0, L, L)
        Z = W + lam * D.dot(D.transpose())
        z = spsolve(Z, w * y)
        w = p * (y > z) + (1 - p) * (y < z)
    return z

def GetFunc(clist):
    def func3(x, a1, a2, a3, s1, s2, s3):
        gaussian_1 = a1 * np.exp(-((x - clist[0]) / s1) ** 2)
        gaussian_2 = a2 * np.exp(-((x - clist[1]) / s2) ** 2)
        gaussian_3 = a3 * np.exp(-((x - clist[2]) / s3) ** 2)
        return gaussian_1 + gaussian_2 + gaussian_3

    return func3


# Main peakfit class
class Amid1Fit:
    def __init__(self, input_dir, output_dir, exclude_area=True, exclude_trash=False, maxerror=0.07):
        if input_dir[-4:] != '.csv':
            raise ValueError('Please select a csv file.')
        if output_dir[-4:] != '.csv':
            output_dir = output_dir + '.csv'
        self._input_dir = input_dir
        self._output_dir = output_dir
        self._exclude_area = exclude_area
        self._exclude_trash = exclude_trash
        self._maxerror = maxerror

    def __call__(self, parallel=False):
        if parallel:
            if self._exclude_area:
                area = ''
            else:
                area = ' -a'
            if self._exclude_trash:
                etrash = ' -e'
            else:
                etrash = ''
            os.system('{} amid1_mpfit.py{}{} -i {} -o {} -p {} --maxerror {}'.format(sys.executable,
                                                                        etrash, area, self._input_dir, self._output_dir,
                                                                        parallel, self._maxerror))
        else:
            # Fitting method
            input_data = []
            with open(self._input_dir) as f:
                reader = csv.reader(f)
                for row in reader:
                    input_data.append(row)
            input_data = np.array(input_data)
            axis_x, spectra = TruncateRange(input_data[1:, 1:].astype(float),
                                            input_data[0, 1:].astype(float),
                                            RangeConstructor(1720, 1600))
            if self._exclude_trash:
                if 'tellamide' not in dir():
                    from tellamide import model, tellamide
                    global model, tellamide
                alist = tellamide(input_data[1:, 1:].astype(float), input_data[0, 1:].astype(float))
            list1 = []
            list2 = []
            list3 = []
            list4 = []
            list5 = []
            list6 = []

            if self._exclude_trash:

                for i in range(len(spectra)):
                    if alist[i]:
                        clist = DetermineAmid1Centroids(FindPeakCentroids(axis_x, spectra[i]))
                        lr = fit(axis_x, spectra[i].astype(float), clist, self._maxerror)
                    else:
                        lr = [0, 0, 0, 0, 0, 0]
                    list1.append(lr[0])
                    list2.append(lr[1])
                    list3.append(lr[2])
                    list4.append(lr[3])
                    list5.append(lr[4])
                    list6.append(lr[5])

            else:
                for i in range(len(spectra)):
                    clist = DetermineAmid1Centroids(FindPeakCentroids(axis_x, spectra[i]))
                    lr = fit(axis_x, spectra[i], clist, self._maxerror)
                    list1.append(lr[0])
                    list2.append(lr[1])
                    list3.append(lr[2])
                    list4.append(lr[3])
                    list5.append(lr[4])
                    list6.append(lr[5])

            list_total = np.array([list1, list2, list3, list4, list5, list6]).transpose()

            # Filename control
            if os.path.exists(self._output_dir):
                i = 1
                output_file = self._output_dir[:-4] + '(' + str(i) + ').csv'
                while os.path.exists(output_file):
                    i += 1
                    output_file = self._output_dir[:-4] + '(' + str(i) + ').csv'
                self._output_dir = output_file

            with open(self._output_dir, 'w', newline='') as f:
                writer = csv.writer(f)
                if self._exclude_area:
                    writer.writerow(['#', 'bsheet', 'rcoil', 'bturn'])
                    for i, row in enumerate(list_total):
                        writer.writerow([i + 1, row[3], row[4], row[5]])
                else:
                    writer.writerow(['#', 'Absheet', 'Arcoil', 'Abturn', 'bsheet', 'rcoil', 'bturn'])
                    for i, row in enumerate(list_total):
                        writer.writerow([i + 1] + row.tolist())


def FindPeakCentroids(axis_x, axis_y):
    # Use the second derivative method for peak detection
    d3 = np.diff(axis_y, n=3)

    peaks_up = []
    # Temporarily record d3 positions
    last = d3[0] < 0
    for i, value in enumerate(d3):
        peak_entering = value < 0
        if last != peak_entering:
            if not peak_entering:
                peaks_up.append(i)
        last = peak_entering

    # Project the index onto the wavenumber axis
    peaks_up_ticks = [(axis_x[i + 1], axis_x[i + 2]) for i in peaks_up]
    peaks_up_values = [(d3[i - 1], d3[i]) for i in peaks_up]
    peaks_up = [(peaks_up_ticks[i][0] * peaks_up_values[i][1] - peaks_up_ticks[i][1] * peaks_up_values[i][0]) \
                / (peaks_up_values[i][1] - peaks_up_values[i][0]) for i in range(len(peaks_up))]
    return peaks_up


def DetermineAmid1Centroids(clist):
    # Take the output from FindPeakCentroids() and find ones feasible for fitting three peaks for silk
    funout = np.array([clist for i in range(3)])
    std = np.array([[1625, 1650, 1695] for i in clist]).transpose()
    assessment = np.abs(funout - std)
    result = np.argmin(assessment, axis=1)
    if len(result) != len(np.unique(result)):
        print('Warning@DetermineAmid1Centroids: identical entries emerged. Reset centroids to 1625, 1650 and 1695.')
        return [1625, 1650, 1695]
    return [clist[i] for i in result]


def fit(axis_x, axis_y, clist, maxerror):
    # zs = savgol_filter(test_2, 9, 3)  # window size 9, polynomial order 3
    # clist <- List of three peak centroids
    cell = baseline_als(axis_y)
    sample = axis_y - cell
    max_sample = np.max(sample)
    popt, pcov = curve_fit(GetFunc(clist), axis_x, sample, bounds=([0, 0, 0, 0, 0, 0],
                                                                   [np.inf, np.inf, np.inf, np.inf, np.inf, np.inf]),
                           p0=[max_sample, max_sample, max_sample, 5, 10, 5],
                           maxfev=500000)
    a, a1 = quad(lambda x: popt[0] * np.exp(-((x - clist[0]) / popt[3]) ** 2), 1600, 1700)
    b, b1 = quad(lambda x: popt[1] * np.exp(-((x - clist[1]) / popt[4]) ** 2), 1600, 1720)
    c, c1 = quad(lambda x: popt[2] * np.exp(-((x - clist[2]) / popt[5]) ** 2), 1600, 1720)
    rmse = np.sqrt(np.sum((sample - (popt[0] * np.exp(-((axis_x - clist[0]) / popt[3]) ** 2)
                                     + popt[1] * np.exp(-((axis_x - clist[1]) / popt[4]) ** 2)
                                     + popt[2] * np.exp(-((axis_x - clist[2]) / popt[5]) ** 2))) ** 2)) \
           / np.sum(sample)
    if rmse > maxerror:
        return a, b, c, 0, 0, 0
    b_sheet = a / (a + b + c)
    random = b / (a + b + c)
    b_turn = c / (a + b + c)
    return a, b, c, b_sheet, random, b_turn
