# This script defines the main program of predicting contents of beta-sheet and other phases by deep learning models.
# The model has structure looks like Inception model, which is a popular model for image recognition.

import os
import numpy as np
from stdspect import *

import tensorflow.keras.backend as K
from tensorflow.keras.models import Model, load_model
import csv
from stdspect import *


class VersionError(Exception):
    pass


class Amid1Pred:
    def __init__(self, input_dir=None, output_dir=None, model=0, trash=None):
        # Starting from 0.3.0 version, the function will automatically parse the input.
        # input_dir -> Directory of the input file
        # The format of the input file should be:
        # wavenumber 4000.000    3999.000    ...     501.000
        # 1          intensity   intensity   ...     intensity
        # 2          intensity   intensity   ...     intensity
        # ...
        # output_dir -> Directory of the output file
        self._input_dir = input_dir
        self._output_dir = output_dir
        self._model = model
        self._input_tensor = None
        self._input_vernier = None
        if self._input_dir[-4:] != '.csv':
            raise ValueError('File type not supported.')

    def get_input_tensor(self):

        # Load dataset
        input_data = []
        with open(self._input_dir) as f:
            reader = csv.reader(f)
            self._input_vernier = np.array(next(reader)[1:], dtype=np.float32)
            for i, row in enumerate(reader):
                input_data.append(row[1:])

            self._input_tensor = np.array(input_data, dtype=float)

    def change_model(self, new_model):
        self._model = new_model

    def find_and_delete_trash(self, output_file=None):
        # Find and delete trash spectra at one step
        raise VersionError('Deprecated after version 0.3.0: find_and_delete_trash.')

    def find_trash(self):
        # Find and return all trash spectra
        raise VersionError('Deprecated after version 0.3.0: find_trash.')

    def delete_trash(self, output_file=None):
        # Create a new file with all trash spectra removed
        # Note that the order is changed
        raise VersionError('Deprecated after version 0.3.0: delete_trash.')

    def __call__(self, input_dir=None, exclude_trash=True, model=None):
        # Main program of predicting
        if input_dir:
            self._input_dir = input_dir
        if model:
            self._model = model
        self.get_input_tensor()

        input_data = lp(self._input_tensor, vernier=self._input_vernier,
                        new_vernier=RangeConstructor(1720.192, 1604, -1.928)).reshape(-1, 61, 1, 1)

        # Define weighted MSE loss
        def wmse(weights):
            weights = K.variable(weights)

            def loss(y_true, y_pred):
                result = (y_true - y_pred) ** 2
                result = weights * result
                result = K.clip(result, K.epsilon(), 4000 - K.epsilon())
                result = K.sum(result)
                return result

            return loss

        model = load_model('models/' + str(self._model) + '.h5', custom_objects={
            'loss': wmse(np.array([5.296373783030799 ** 2, 1, 13.389198115601468 ** 2]))})

        # Make predictions
        pred = model.predict(input_data)

        if exclude_trash == True:
            excluderinput = lp(self._input_tensor, vernier=self._input_vernier,
                               new_vernier=RangeConstructor(1750, 1200, -1.5)).reshape(-1, 367, 1, 1)
            excludemodel = load_model('models/TellAmide_pretrain.h5')
            excludepred = excludemodel.predict([excluderinput[:, 65:105].reshape(-1, 40, 1, 1),
                                                excluderinput[:, 342:367].reshape(-1, 25, 1, 1)])
            for i,b in enumerate(excludepred):
                if b[0] < 0.5:
                    pred[i] = np.zeros(3)

        # Save predictions to file
        if not self._output_dir:
            filehead = 'Prediction_of_'
            output_name = filehead + self._input_dir[:-4] + '.csv'
            i = 1
            while os.path.exists(output_name):
                output_name = filehead + self._input_dir[:-4] + '({}).csv'.format(i)
                i += 1
            self._output_dir = output_name

        else:
            output_name = self._output_dir
            if self._output_dir[-4:] != '.csv':
                output_name = self._output_dir + '.csv'

            i = 1
            while os.path.exists(output_name):
                output_name = self._output_dir[:-4] + '(' + str(i) + ').csv'
                i += 1
            self._output_dir = output_name

        # os.mknod(self._output_dir)
        # Important warning: NO mknod on WINDOWS

        # Write into a csv file
        with open(self._output_dir, 'w', newline='') as f:  # newline is needed in windows
            writer = csv.writer(f)
            writer.writerow(['#', 'bsheet', 'rcoil', 'bturn'])
            for i, row in enumerate(pred):
                writer.writerow([i + 1] + row.tolist())

if __name__ == '__main__':
    print('Lib for amid1 prediction. Version 0.3.2')
    print('Jul 14, 2020')
    input()
