# This script applies k-means clustering method to tell different phases apart from spectrum data

import numpy as np
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
from sklearn.cluster import KMeans
from sklearn.decomposition import PCA
import os
import csv

class Clustering:
    def __init__(self, input_dir, output_dir=None, cats=2):
        # cats <- number of categories
        self._input_dir = input_dir
        self._output_dir = output_dir
        self._cats = cats
        self._data = None
        self._clust_result = None
        self._pca_result = None
        self._pca_executor = None

    def getdata(self):
        # Get data points from the sheet
        # Load datasets
        input_data = []

        # Input dataset
        with open(self._input_dir) as f:
            reader = csv.reader(f)
            for i, row in enumerate(reader):
                # Discard the first row
                if i == 0:
                    continue
                # Extract essential data
                input_data.append(row[1:])
            input_data = np.array(input_data, dtype=float)

        self._data = input_data

    def clustering(self):
        # Get clustering executor and do the clustering
        if self._data is None:
            raise ValueError('Data not imported. Use getdata() first.')
        clust = KMeans(n_clusters=self._cats, random_state=0)
        self._clust_result = clust.fit_predict(self._data)

    def creat_dir(self):
        # Creat a directory to save all the outputs
        # Make a directory
        if self._output_dir:
            output_dir = self._output_dir
            i = 1
            while os.path.exists(output_dir):
                output_dir = self._output_dir + '({})'.format(i)
                i += 1
            self._output_dir = output_dir
        else:
            raise ValueError('Please specify an output directory name.')
        os.mkdir(self._output_dir)

    def pca(self):
        # PCA analysis to the spectra
        # Get the PCA executor
        if self._data is None:
            raise ValueError('Data not imported. Use getdata() first.')
        self._pca_executor = PCA(n_components=2)
        self._pca_result = self._pca_executor.fit_transform(self._data)

    def plot_clusters(self):
        # Cast the cluster result onto the plain of 2 principle components

        # Find the indeces of each cluster
        c_index = [[] for i in range(self._cats)]

        for i, value in enumerate(self._clust_result):
            c_index[value].append(i)

        c = [[] for i in range(self._cats)]

        for i in range(self._cats):
            for j in c_index[i]:
                c[i].append(self._pca_result[j])

        for pcr in c:
            pcr = np.array(pcr)
            plt.scatter(pcr[:, 0], pcr[:, 1], s=3)
        plt.title('Visualization on PC Plane')
        plt.legend(['Cluster {}'.format(i) for i in range(self._cats)], loc="upper left")
        plt.savefig(self._output_dir + '/Cluster_Visualization.png', dpi=300)
        plt.close()

    def output_clusters(self):
        # Write the clusters to files
        with open(self._output_dir + '/Cluster.csv', 'w', newline='') as f:
            writer = csv.writer(f)
            writer.writerow(['#', 'Cluster #'])
            for i, value in enumerate(self._clust_result):
                writer.writerow([i+1, value])

    def plot_eigen(self):
        # Plot the eigenvectors
        with open(self._input_dir) as f:
            # First get the X-axis values
            vernier = np.array(f.readline().split(',')[1:], dtype=float)
        for eigenv in self._pca_executor.components_:
            plt.plot(vernier, eigenv)
        plt.gca().invert_xaxis()
        plt.title('Eigen Spectra')
        plt.legend(['PC 1','PC 2']) # Always only do the first 2 PCs
        plt.savefig(self._output_dir+'/PC_Eigenvectors.png', dpi=300)
        plt.close()
