# This script reconstructs the 2D phase distribution

import numpy as np
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import csv
import os
from scipy.io import savemat

# Load alphabetic order list


class Phase2D:
    def __init__(self, input_dir, output_dir=None, order='r', shape=(64, 64), plot=True, mat=True):
        # Defines the input and output directory
        # "input_dir" must be a csv file
        # "output_dir" must be a directory
        # "order" is responsable for correcting input order of the data
        # For example:
        # order='regular' or 'r' => 1 to (0, 0), 2 to (0, 1), 3 to (0, 2), ...
        # order='alphabetic' or 'a' => 1 to (0, 0), 10 to (0, 1), 100 to (0, 2), ...
        # "shape" refers to the shape of the reconstructed distribution map, whose product
        # needs to equal to the number of spectra
        self._input_dir = input_dir
        self._output_dir = output_dir
        self._order = order
        self._shape = shape
        self._data = None
        self._labels = None
        self._reconstructed = None
        self._reordered = False
        self._maps = None
        self._plot = plot  # Whether output plots
        self._mat = mat  # Whether output mat file

    def getdata(self):
        # Take in the phase distribution data
        with open(self._input_dir) as f:
            reader = csv.reader(f)
            self._data = []
            for i, row in enumerate(reader):
                if i == 0:
                    self._labels = row[1:]
                else:
                    self._data.append(row)

        self._data = np.array(self._data, dtype=float)

    def isvalid(self):
        if type(self._data) is not None:
            if len(self._data) == self._shape[0] * self._shape[1]:
                return True
            else:
                raise ValueError(
                    'Shape ({}*{}) does not match number of spectra ({}).'.format(self._shape[0], self._shape[1],
                                                                                  len(self._data)))
        else:
            raise ValueError('Data not imported. Use getdata() first.')

    def reorder(self):
        # Reorder the data from the sheet so that reconstructing by row becomes easy
        if self._reordered:
            print('Already reordered.')
            return
        if self._data is not None:
            if self._order in ['r', 'regular']:
                self._data = self._data[np.argsort(self._data[:, 0])]
                self._reordered = True
            elif self._order in ['a', 'alphabetic']:
                order = [str(i) for i in range(len(self._data))]
                order.sort()
                order = np.array(order, dtype=int).reshape(-1, 1)
                self._data = np.concatenate(
                    [order] + [self._data[:, i].reshape(-1, 1) for i in range(self._data.shape[1])], axis=1)
                self._data = self._data[np.argsort(self._data[:, 0])]
                self._data = np.delete(self._data, 1, axis=1)
                self._reordered = True
            else:
                raise AttributeError('order "{}" not found'.format(self._order))

    def mapmake(self):
        # Set up the matrix of 2D distribution
        # Return None, self._maps set to a dict with maps labeled by phase composition
        # e.g. self._maps = {'bsheet': 64*64 np.array, 'rcoil': 64*64 np.array}
        self._maps = {}
        for i in range(self._data.shape[1]):
            if i == 0:
                continue
            cur_map = self._data[:, i].reshape(self._shape[0], self._shape[1])
            self._maps[self._labels[i - 1]] = cur_map

    def showmap(self, label):
        # Show the specific constructed map
        plt.imshow(self._maps[label])
        plt.colorbar()
        plt.show()

    def savemap(self):
        # Make a directory
        if self._output_dir:
            output_dir = self._output_dir
            i = 1
            while os.path.exists(output_dir):
                output_dir = self._output_dir + '({})'.format(i)
                i += 1
            self._output_dir = output_dir
        else:
            raise ValueError('Please specify an output directory name.')
        os.mkdir(self._output_dir)

        if self._plot:
            # Save drawn images to a directory
            for key in self._maps:
                plt.imshow(self._maps[key])
                plt.colorbar()
                plt.savefig(self._output_dir + '/' + key + '.png', dpi=300)
                plt.close()

        if self._mat:
            # Save as matrix format
            outmat_name = self._output_dir + '/2D.mat'
            savemat(outmat_name, self._maps)

    def __call__(self):
        # Execute the full flow from getdata() through mapmake()
        self.getdata()
        self.isvalid()
        self.reorder()
        self.mapmake()
        self.savemap()
